<?php
/**
 * @file
 * Helper functions for the Met Office module.
 */


/**
 * Populate the DB table with the list of Met Office sites.
 *
 * @param Boolean $force_rebuild
 * Set to TRUE to ignore the age of the site-list cache and rebuild anyway.
 *
 */
function metoffice_populate_sitelist($force_rebuild = FALSE) {
  // Check how recent the cache is.
  $cache_age = REQUEST_TIME - variable_get('metoffice_site_cache_last_built', 0);
  $cache_is_expired = ($cache_age > variable_get('metoffice_site_cache_max_age', METOFFICE_DEFAULT_SITE_CACHE_MAX_AGE));

  // If the cache is out of date (or a new build is forced), rebuild the cache.
  if ($cache_is_expired || $force_rebuild) {
    $sites = metoffice_lookup_sitelist_using_api();
    if ($sites) {

      // Truncate the site-cache table, before adding the updated site list.
      db_truncate('metoffice_site_cache')->execute();

      foreach ($sites as $site) {
        db_insert('metoffice_site_cache')
          ->fields((array) $site)
          ->execute();
      }
    }
    variable_set('metoffice_site_cache_last_built', REQUEST_TIME);
    watchdog('Met Office', 'Site-list cache rebuilt.', array(), WATCHDOG_INFO);
  }
}

/**
 * Lookup the list of sites directly from the Met Office API.
 *
 * @return Array
 * Array of site locations, keyed by the Met Office id of the location.
 * Each location has the properties:
 * - id
 * - longitude
 * - latitude
 * - name
 * If the request does not complete, an empty array is returned, and the error is
 * logged.
 */
function metoffice_lookup_sitelist_using_api() {
  $data = _metoffice_api_query(METOFFICE_URL_SITELIST);

  $sites = array();
  // The Met Office API returns the locations in a JSON structure with
  // Locations and Location as root elements.
  foreach ($data->Locations->Location as $site) {
    $sites[$site->id] = $site;
  }

  return $sites;
}


/**
 * Query the API.
 *
 * @param String $url
 * URL of the API.
 * @param Array $query
 * Options to pass as API parameters.
 *
 * @return Mixed
 * Response from the API.
 */
function _metoffice_api_query($url, $query = array()) {
  // Quick check that the API key is populated.
  $api_key = variable_get('metoffice_apikey', '');
  if (empty($api_key)) {
    watchdog('Met office', 'Unable to query web service - API key not set.', array(), WATCHDOG_ERROR);
    return array();
  }

  // Merge the API key with the query parameters.
  $query['key'] = $api_key;
  $url = url($url, array('query' => $query));

  $response = drupal_http_request($url, array(
    'headers' => array('User-Agent' => METOFFICE_HTTP_UA),
  ));

  if (!empty($response->error)) {
    watchdog('Met office', 'Web service failed: %error_message.', array('%error_message' => $response->error), WATCHDOG_ERROR);
    return array();
  }

  // Make sure the data is UTF8-encoded.
  // See http://stackoverflow.com/questions/689185/json-decode-returns-null-php
  // Without this, json_decode silently fails (without reporting any errors).
  $json = utf8_encode($response->data);
  $data = json_decode($json);

  if (empty($data)) {
    watchdog('Met office', 'Web service failed: JSON response could not be decoded.', array(), WATCHDOG_ERROR);
    return array();
  }

  return $data;
}








// /**
//  * Lookup the list of sites directly from the Met Office API.
//  *
//  * @return Array
//  * Array of site locations, keyed by the Met Office id of the location.
//  * Each location has the properties:
//  * - id
//  * - longitude
//  * - latitude
//  * - name
//  * If the request does not complete, an empty array is returned, and the error is
//  * logged.
//  */
// function metoffice_lookup_sitelist_using_api() {
//   // Quick check that the API key is populated.
//   $api_key = variable_get('metoffice_apikey', '');
//   if (empty($api_key)) {
//     watchdog('Met office', 'Unable to query web service - API key not set.', array(), WATCHDOG_ERROR);
//     return array();
//   }
// 
//   $url = url(METOFFICE_URL_SITELIST, array(
//     'query' => array(
//       'key' => $api_key,
//     ),
//   ));
// 
//   $response = drupal_http_request($url);
// 
//   if (!empty($response->error)) {
//     watchdog('Met office', 'Web service failed: %error_message.', array('%error_message' => $response->error), WATCHDOG_ERROR);
//     return array();
//   }
// 
//   // Make sure the data is UTF8-encoded.
//   // See http://stackoverflow.com/questions/689185/json-decode-returns-null-php
//   // Without this, json_decode silently fails (without reporting any errors).
//   $json = utf8_encode($response->data);
//   $data = json_decode($json);
// 
//   if (empty($data)) {
//     watchdog('Met office', 'Web service failed: JSON response could not be decoded.', array(), WATCHDOG_ERROR);
//     return array();
//   }
// 
//   $sites = array();
//   // The Met Office API returns the locations in a JSON structure with
//   // Locations and Location as root elements.
//   foreach ($data->Locations->Location as $site) {
//     $sites[$site->id] = $site;
//   }
// 
//   return $sites;
// }
